@@ -1,6 +1,7 @@
 /*
  * gnote
  *
+ * Copyright (C) 2010-2011 Aurimas Cernius
  * Copyright (C) 2010 Debarshi Ray
  * Copyright (C) 2009 Hubert Figuiere
  *
@@ -340,7 +341,8 @@ namespace gnote {
     about.set_name("Gnote");
     about.set_version(VERSION);
     about.set_logo(utils::get_icon("gnote", 48));
-    about.set_copyright(_("Copyright \xc2\xa9 2009-2010 Debarshi Ray\n"
+    about.set_copyright(_("Copyright \xc2\xa9 2010-2011 Aurimas Cernius\n"
+                          "Copyright \xc2\xa9 2009-2011 Debarshi Ray\n"
                           "Copyright \xc2\xa9 2009 Hubert Figuiere\n"
                           "Copyright \xc2\xa9 2004-2009 the Tomboy original authors."));
     about.set_comments(_("A simple and easy to use desktop "
@@ -543,13 +545,7 @@ namespace gnote {
       m_open_note_uri = remote->FindNote (m_open_note_name);
     }
     if (!m_open_note_uri.empty()) {
-      if (m_highlight_search) {
-        remote->DisplayNoteWithSearch (m_open_note_uri,
-                                       m_highlight_search);
-      }
-      else {
-        remote->DisplayNote (m_open_note_uri);
-      }
+      display_note(remote, m_open_note_uri);
     }
 
     if (!m_open_external_note_path.empty()) {
@@ -557,9 +553,7 @@ namespace gnote {
       if (!note_id.empty()) {
         // Attempt to load the note, assuming it might already
         // be part of our notes list.
-        if (remote->DisplayNote (
-              str(boost::format("note://gnote/%1%") % note_id)) == false) {
-
+        if (!display_note(remote, str(boost::format("note://gnote/%1%") % note_id))) {
           sharp::StreamReader sr;
           sr.init(m_open_external_note_path);
           if (sr.file()) {
@@ -576,7 +570,7 @@ namespace gnote {
               noteXml = "";
             }
 
-						if (!noteXml.empty()) {
+            if (!noteXml.empty()) {
               noteTitle = NoteArchiver::obj().get_title_from_note_xml (noteXml);
               if (!noteTitle.empty()) {
                 // Check for conflicting titles
@@ -593,7 +587,7 @@ namespace gnote {
                 if (!note_uri.empty()) {
                   // Load in the XML contents of the note file
                   if (remote->SetNoteCompleteXml (note_uri, noteXml)) {
-                    remote->DisplayNote (note_uri);
+                    display_note(remote, note_uri);
                   }
                 }
               }
@@ -639,6 +633,22 @@ namespace gnote {
   }
 
 
+  bool GnoteCommandLine::display_note(RemoteControlClient * remote,
+                                      std::string uri)
+  {
+#ifdef ENABLE_DBUS
+    if (m_highlight_search) {
+      return remote->DisplayNoteWithSearch(uri, m_highlight_search);
+    }
+    else {
+      return remote->DisplayNote (uri);
+    }
+#else
+    return false;
+#endif
+  }
+
+
   bool GnoteCommandLine::needs_execute() const
   {
     DBG_OUT("needs execute?");
