@@ -1,6 +1,8 @@
 /*
  * gnote
  *
+ * Copyright (C) 2010-2011 Aurimas Cernius
+ * Copyright (C) 2010 Debarshi Ray
  * Copyright (C) 2009 Hubert Figuiere
  *
  * This program is free software: you can redistribute it and/or modify
@@ -17,6 +19,9 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 
 #include <string.h>
 
@@ -24,8 +29,6 @@
 
 #include <boost/bind.hpp>
 #include <boost/format.hpp>
-#include <boost/filesystem/operations.hpp>
-#include <boost/filesystem/path.hpp>
 
 #include <glib.h>
 #include <glibmm/i18n.h>
@@ -45,6 +48,7 @@
 #include "sharp/uuid.hpp"
 #include "sharp/string.hpp"
 #include "sharp/datetime.hpp"
+#include "notebooks/notebookmanager.hpp"
 
 namespace gnote {
 
@@ -104,7 +108,7 @@ namespace gnote {
     m_start_note_uri = prefs.get<std::string>(Preferences::START_NOTE_URI);
     prefs.signal_setting_changed().connect(
       sigc::mem_fun(*this, &NoteManager::on_setting_changed));
-    m_note_template_title = _("New Note Template");
+    m_default_note_template_title = _("New Note Template");
 
 
     DBG_OUT("NoteManager created with note path \"%s\".", directory.c_str());
@@ -192,15 +196,13 @@ namespace gnote {
   // For overriding in test methods.
   bool NoteManager::directory_exists(const std::string & directory) const
   {
-    boost::filesystem::path p(directory);
-    return exists(p) && is_directory(p);
+    return sharp::directory_exists(directory);
   }
 
   // For overriding in test methods.
   bool NoteManager::create_directory(const std::string & directory) const
   {
-    boost::filesystem::path p(directory);
-    return boost::filesystem::create_directory(p);
+    return g_mkdir_with_parents(directory.c_str(), S_IRWXU) == 0;
   }
 
   bool NoteManager::first_run() const
@@ -433,21 +435,21 @@ namespace gnote {
 
   void NoteManager::delete_note(const Note::Ptr & note)
   {
-    if (boost::filesystem::exists(note->file_path())) {
+    if (sharp::file_exists(note->file_path())) {
       if (!m_backup_dir.empty()) {
-        if (!boost::filesystem::exists(m_backup_dir)) {
-          boost::filesystem::create_directory(m_backup_dir);
+        if (!sharp::directory_exists(m_backup_dir)) {
+          sharp::directory_create(m_backup_dir);
         }
         std::string backup_path 
           = Glib::build_filename(m_backup_dir, sharp::file_filename(note->file_path()));
-          
-        if (boost::filesystem::exists(backup_path))
-          boost::filesystem::remove(backup_path);
 
-        boost::filesystem::rename(note->file_path(), backup_path);
+        if (sharp::file_exists(backup_path))
+          sharp::file_delete(backup_path);
+
+        sharp::file_move(note->file_path(), backup_path);
       } 
       else {
-        boost::filesystem::remove(note->file_path());
+        sharp::file_delete(note->file_path());
       }
     }
 
@@ -472,18 +474,7 @@ namespace gnote {
 
   Note::Ptr NoteManager::create()
   {
-    int new_num = m_notes.size();
-    std::string temp_title;
-    
-    while (true) {
-      ++new_num;
-      temp_title = str(boost::format(_("New Note %1%")) %  new_num);
-      if (!find(temp_title)) {
-        break;
-      }
-    }
-
-    return create(temp_title);
+    return create("");
   }
 
   std::string NoteManager::split_title_from_content(std::string title, std::string & body)
@@ -553,49 +544,49 @@ namespace gnote {
     return create_new_note(title, guid);
   }
 
-  // Create a new note with the specified title, and a simple
-  // "Describe..." body or the body from the "New Note Template"
-  // note if it exists.  If the "New Note Template" body is found
-  // the text will not automatically be highlighted.
+  // Create a new note with the specified title from the default
+  // template note. Optionally the body can be overridden.
   Note::Ptr NoteManager::create_new_note (std::string title, const std::string & guid)
   {
     std::string body;
-    
     title = split_title_from_content (title, body);
-    if (title.empty())
-      return Note::Ptr();
       
-    Note::Ptr note_template = find(m_note_template_title);
-    if (note_template) {
-      // Use the body from the "New Note Template" note
-      std::string xml_content =
-        sharp::string_replace_first(note_template->xml_content(), 
-                                    m_note_template_title,
-                                    utils::XmlEncoder::encode (title));
-      return create_new_note (title, xml_content, guid);
+    if (title.empty()) {
+      title = get_unique_name(_("New Note"), m_notes.size());
     }
-      
-    // Use a simple "Describe..." body and highlight
-    // it so it can be easily overwritten
-    body = _("Describe your new note here.");
-    
+
+    if (body.empty()) {
+      std::string content = get_note_template_content(title);
+      Note::Ptr new_note = create_new_note (title, content, guid);
+      new_note->get_buffer()->select_note_body();
+      // Use the body from the template note
+      Note::Ptr template_note = find_template_note();
+      if(template_note)
+        replace_body_if_differ(new_note, template_note);
+      return new_note;
+    }
+
     Glib::ustring header = title + "\n\n";
     std::string content =
       boost::str(boost::format("<note-content>%1%%2%</note-content>") %
                  utils::XmlEncoder::encode (header) 
                  % utils::XmlEncoder::encode (body));
-    
-    Note::Ptr new_note = create_new_note (title, content, guid);
-    
-    // Select the inital
-    // "Describe..." text so typing will overwrite the body text,
-    //NoteBuffer 
-    Glib::RefPtr<Gtk::TextBuffer> buffer = new_note->get_buffer();
-    Gtk::TextIter iter = buffer->get_iter_at_offset(header.size());
-    buffer->move_mark (buffer->get_selection_bound(), iter);
-    buffer->move_mark (buffer->get_insert(), buffer->end());
-    
-    return new_note;
+    return create_new_note (title, content, guid);
+  }
+
+  //replace dest body by src one, if the text is different
+  void NoteManager::replace_body_if_differ(Note::Ptr dest, const Note::Ptr src)
+  {
+    std::string dest_body = sharp::string_trim(sharp::string_replace_first(dest->text_content(),
+                                                                           dest->get_title(), ""));
+    std::string src_body = sharp::string_trim(sharp::string_replace_first(src->text_content(),
+                                                                          src->get_title(), ""));
+    if(dest_body != src_body) {
+      std::string xml_content = sharp::string_replace_first(src->xml_content(),
+        sharp::string_trim(utils::XmlEncoder::encode(src->get_title())),
+        sharp::string_trim(utils::XmlEncoder::encode(dest->get_title())));
+      dest->set_xml_content(xml_content);
+    }
   }
 
   // Create a new note with the specified Xml content
@@ -629,6 +620,26 @@ namespace gnote {
     return new_note;
   }
 
+  Note::Ptr NoteManager::find_template_note() const
+  {
+    Note::Ptr template_note;
+    Tag::Ptr template_tag = TagManager::obj().get_system_tag (TagManager::TEMPLATE_NOTE_SYSTEM_TAG);
+    if(!template_tag) {
+      return template_note;
+    }
+    std::list<Note*> notes;
+    template_tag->get_notes(notes);
+    for (std::list<Note*>::iterator iter = notes.begin(); iter != notes.end(); ++iter) {
+      Note::Ptr note = (*iter)->shared_from_this();
+      if (!notebooks::NotebookManager::instance().get_notebook_from_note (note)) {
+        template_note = note;
+        break;
+      }
+    }
+
+    return template_note;
+  }
+
   /// <summary>
   /// Get the existing template note or create a new one
   /// if it doesn't already exist.
@@ -638,21 +649,23 @@ namespace gnote {
   /// </returns>
   Note::Ptr NoteManager::get_or_create_template_note()
   {
-    Note::Ptr template_note = find(m_note_template_title);
+    Note::Ptr template_note = find_template_note();
     if (!template_note) {
+      std::string title = m_default_note_template_title;
+      if (find(title)) {
+        title = get_unique_name(title, m_notes.size());
+      }
       template_note =
-        create (m_note_template_title,
-                get_note_template_content(m_note_template_title));
+        create (title,
+                get_note_template_content(title));
           
       // Select the initial text
       Glib::RefPtr<NoteBuffer> buffer = template_note->get_buffer();
-      Gtk::TextIter iter = buffer->get_iter_at_line_offset(2, 0);
-      buffer->move_mark(buffer->get_selection_bound(), iter);
-      buffer->move_mark(buffer->get_insert(), buffer->end());
+      buffer->select_note_body();
 
       // Flag this as a template note
-      Tag::Ptr tag = TagManager::obj().get_or_create_system_tag(TagManager::TEMPLATE_NOTE_SYSTEM_TAG);
-      template_note->add_tag(tag);
+      Tag::Ptr template_tag = TagManager::obj().get_or_create_system_tag (TagManager::TEMPLATE_NOTE_SYSTEM_TAG);
+      template_note->add_tag(template_tag);
 
       template_note->queue_save(Note::CONTENT_CHANGED);
     }
@@ -704,6 +717,21 @@ namespace gnote {
     return Note::Ptr();
   }
 
+  // Find a title that does not exist using basename and id as
+  // a starting point
+  std::string NoteManager::get_unique_name (std::string basename, int id) const
+  {
+    std::string title;
+    while (true) {
+      title = str(boost::format("%1% %2%") % basename % id++);
+      if (!find (title)) {
+        break;
+      }
+    }
+
+    return title;
+  }
+
 
   TrieController::TrieController (NoteManager & manager)
     : m_manager(manager)
