@@ -1,6 +1,7 @@
 /*
  * gnote
  *
+ * Copyright (C) 2011 Aurimas Cernius
  * Copyright (C) 2009 Hubert Figuiere
  *
  * This program is free software: you can redistribute it and/or modify
@@ -28,6 +29,9 @@
 #include <gtkmm/toolbar.h>
 #include <gtkmm/toolbutton.h>
 #include <gtkmm/menu.h>
+#include <gtkmm/checkmenuitem.h>
+#include <gtkmm/radiomenuitem.h>
+#include <gtkmm/imagemenuitem.h>
 #include <gtkmm/textview.h>
 #include <gtkmm/scrolledwindow.h>
 
@@ -64,6 +68,8 @@ private:
   void toggle_bullets_clicked();
   void increase_indent_clicked();
   void decrease_indent_clicked();
+  void increase_font_clicked();
+  void decrease_font_clicked();
 
   Glib::RefPtr<NoteBuffer> m_buffer;
   UndoManager          &m_undo_manager;
@@ -85,6 +91,8 @@ private:
   Gtk::CheckMenuItem    m_bullets;
   Gtk::ImageMenuItem    m_increase_indent;
   Gtk::ImageMenuItem    m_decrease_indent;
+  Gtk::MenuItem         m_increase_font;
+  Gtk::MenuItem         m_decrease_font;
   sigc::connection      m_bullets_clicked_cid;
 };
 
@@ -129,7 +137,6 @@ protected:
   virtual bool on_delete_event(GdkEventAny *ev);
   virtual void on_hide();
 private:
-  static void on_escape_setting_changed(GConfClient *, guint cnxid, GConfEntry* entry, gpointer data);
   bool on_key_pressed(GdkEventKey*);
   void close_window_handler();
   void close_all_windows_handler();
@@ -155,7 +162,6 @@ private:
   Note                        & m_note;
   Glib::RefPtr<Gtk::AccelGroup> m_accel_group;
   Gtk::Toolbar                 *m_toolbar;
-  Gtk::Tooltips                *m_toolbar_tips;
   Gtk::ToolButton              *m_link_button;
   NoteTextMenu                 *m_text_menu;
   Gtk::Menu                    *m_plugin_menu;
@@ -166,7 +172,6 @@ private:
 
   utils::GlobalKeybinder       *m_global_keys;
   utils::InterruptableTimeout  *m_mark_set_timeout;
-  sigc::connection              m_keypress_cid;
   guint                         m_gconf_notify;
 };
 
