@@ -1,6 +1,8 @@
 /*
  * gnote
  *
+ * Copyright (C) 2010-2011 Aurimas Cernius
+ * Copyright (C) 2010 Debarshi Ray
  * Copyright (C) 2009 Hubert Figuiere
  *
  * This program is free software: you can redistribute it and/or modify
@@ -17,14 +19,15 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 
 #include <boost/format.hpp>
 
 #include <glibmm/i18n.h>
 #include <gtkmm/image.h>
 #include <gtkmm/main.h>
-#include <gtkmm/paned.h>
 #include <gtkmm/stock.h>
 #include <gtkmm/table.h>
 #include <gtkmm/treestore.h>
@@ -153,8 +156,8 @@ namespace gnote {
     m_tree = manage(m_tree);
     m_tree->show ();
 
-    m_note_count.property_xalign() = 0;
-    m_note_count.show ();
+    m_status_bar.set_has_resize_grip(true);
+    m_status_bar.show();
 
     // Update on changes to notes
     m.signal_note_deleted.connect(sigc::mem_fun(*this, &NoteRecentChanges::on_notes_changed));
@@ -174,28 +177,23 @@ namespace gnote {
     m_tree->get_size_request (h,w);
     m_matches_window.set_size_request(std::min(h, 420), std::min(w, 480));
                         
-    restore_position ();
-
     m_matches_window.property_hscrollbar_policy() = Gtk::POLICY_AUTOMATIC;
     m_matches_window.property_vscrollbar_policy() = Gtk::POLICY_AUTOMATIC;
     m_matches_window.add (*m_tree);
     m_matches_window.show ();
 
-    Gtk::HPaned *hpaned = manage(new Gtk::HPaned ());
-    hpaned->set_position(150);
-    hpaned->add1 (*notebooksPane);
-    hpaned->add2 (m_matches_window);
-    hpaned->show ();
+    m_hpaned.set_position(150);
+    m_hpaned.add1 (*notebooksPane);
+    m_hpaned.add2 (m_matches_window);
+    m_hpaned.show ();
 
-    Gtk::HBox *status_box = manage(new Gtk::HBox (false, 8));
-    status_box->pack_start (m_note_count, true, true, 0);
-    status_box->show ();
+    restore_position ();
 
     Gtk::VBox *vbox = manage(new Gtk::VBox (false, 8));
     vbox->set_border_width(6);
     vbox->pack_start (*hbox, false, false, 0);
-    vbox->pack_start (*hpaned, true, true, 0);
-    vbox->pack_start (*status_box, false, false, 0);
+    vbox->pack_start (m_hpaned, true, true, 0);
+    vbox->pack_start (m_status_bar, false, false, 0);
     vbox->show ();
 
     // Use another VBox to place the MenuBar
@@ -281,13 +279,15 @@ namespace gnote {
     column->set_cell_data_func (*renderer,
                                 sigc::mem_fun(*this, &NoteRecentChanges::notebook_pixbuf_cell_data_func));
       
-    renderer = manage(new Gtk::CellRendererText ());
-    column->pack_start (*renderer, true);
-    column->set_cell_data_func (*renderer,
+    Gtk::CellRendererText *text_renderer = manage(new Gtk::CellRendererText ());
+    text_renderer->property_editable() = true;
+    column->pack_start (*text_renderer, true);
+    column->set_cell_data_func (*text_renderer,
                                 sigc::mem_fun(*this, &NoteRecentChanges::notebook_text_cell_data_func));
-      
+    text_renderer->signal_edited().connect(sigc::mem_fun(*this, &NoteRecentChanges::on_notebook_row_edited));
+
     m_notebooksTree->append_column (*column);
-      
+
     m_notebooksTree->signal_row_activated()
       .connect(sigc::mem_fun(*this, &NoteRecentChanges::on_notebook_row_activated));
     m_on_notebook_selection_changed_cid = m_notebooksTree->get_selection()->signal_changed()
@@ -335,6 +335,9 @@ namespace gnote {
       sigc::mem_fun(*this, &NoteRecentChanges::on_treeview_motion_notify), false);
     m_tree->signal_button_release_event().connect(
       sigc::mem_fun(*this, &NoteRecentChanges::on_treeview_button_released));
+    m_tree->signal_key_press_event().connect(
+      sigc::mem_fun(*this,
+                    &NoteRecentChanges::on_treeview_key_pressed));
     m_tree->signal_drag_data_get().connect(
       sigc::mem_fun(*this, &NoteRecentChanges::on_treeview_drag_data_get));
 
@@ -578,7 +581,9 @@ namespace gnote {
     try {
       const char * fmt;
       fmt = ngettext("Total: %1% note", "Total: %1% notes", total);
-      m_note_count.set_text(str(boost::format (fmt) % total));
+      std::string status = str(boost::format (fmt) % total);
+      m_status_bar.pop(0);
+      m_status_bar.push(status, 0);
     } 
     catch(const std::exception & e)
     {
@@ -592,7 +597,9 @@ namespace gnote {
     try {
       const char * fmt;
       fmt = ngettext("Matches: %1% note", "Matches: %1% notes", matches);
-      m_note_count.set_text(str(boost::format (fmt) % matches));
+      std::string status = str(boost::format (fmt) % matches);
+      m_status_bar.pop(0);
+      m_status_bar.push(status, 0);
     }
     catch(const std::exception & e)
     {
@@ -735,6 +742,34 @@ namespace gnote {
     }
   }
 
+  void NoteRecentChanges::on_notebook_row_edited(const Glib::ustring& tree_path,
+                                                 const Glib::ustring& new_text)
+  {
+    if (notebooks::NotebookManager::instance().notebook_exists(new_text) ||
+        new_text == "") {
+      return;
+    }
+    notebooks::Notebook::Ptr old_notebook = this->get_selected_notebook ();
+    if (std::tr1::dynamic_pointer_cast<notebooks::SpecialNotebook>(old_notebook)) {
+      return;
+    }
+    notebooks::Notebook::Ptr new_notebook = notebooks::NotebookManager::instance()
+      .get_or_create_notebook (new_text);
+    DBG_OUT("Renaming notebook '{%s}' to '{%s}'", old_notebook->get_name().c_str(),
+            new_text.c_str());
+    std::list<Note *> notes;
+    old_notebook->get_tag()->get_notes(notes);
+    for(std::list<Note *>::const_iterator note = notes.begin(); note != notes.end(); ++note) {
+      notebooks::NotebookManager::instance().move_note_to_notebook (
+          (*note)->shared_from_this(), new_notebook);
+    }
+    notebooks::NotebookManager::instance().delete_notebook (old_notebook);
+    Gtk::TreeIter iter;
+    if (notebooks::NotebookManager::instance().get_notebook_iter (new_notebook, iter)) {
+      m_notebooksTree->get_selection()->select(iter);
+    }
+  }
+
   void NoteRecentChanges::on_selection_changed()
   {
     Note::List selected_notes = get_selected_notes ();
@@ -797,6 +832,19 @@ namespace gnote {
       break;
     case GDK_BUTTON_PRESS:
       if (ev->button == 3) {
+        const Glib::RefPtr<Gtk::TreeSelection> selection
+          = m_tree->get_selection();
+
+        if(selection->get_selected_rows().size() <= 1) {
+          Gtk::TreeViewColumn * col = 0; // unused
+          Gtk::TreePath p;
+          int cell_x, cell_y;            // unused
+          if (m_tree->get_path_at_pos(ev->x, ev->y, p, col,
+                                      cell_x, cell_y)) {
+            selection->unselect_all();
+            selection->select(p);
+          }
+        }
         Gtk::Menu *menu = dynamic_cast<Gtk::Menu*>(
           ActionManager::obj().get_widget("/MainWindowContextMenu"));
         popup_context_menu_at_location (menu, ev->x, ev->y);
@@ -885,6 +933,29 @@ namespace gnote {
     return false;
   }
                 
+  bool NoteRecentChanges::on_treeview_key_pressed(GdkEventKey * ev)
+  {
+    switch (ev->keyval) {
+    case GDK_Menu:
+    {
+      // Pop up the context menu if a note is selected
+      Note::List selected_notes = get_selected_notes();
+      if (!selected_notes.empty()) {
+        ActionManager & manager = ActionManager::obj();
+        Gtk::Menu * const menu
+          = dynamic_cast<Gtk::Menu*>(manager.get_widget(
+                                       "/MainWindowContextMenu"));
+        popup_context_menu_at_location(menu, 0, 0);
+      }
+      break;
+    }
+    default:
+      break;
+    }
+
+    return false; // Let Escape be handled by the window.
+  }
+
   void NoteRecentChanges::popup_context_menu_at_location(Gtk::Menu *menu, int x, int y)
   {
     menu->show_all ();
@@ -904,67 +975,44 @@ namespace gnote {
   // the context menu with the keyboard.
   void NoteRecentChanges::position_context_menu(int & x, int & y, bool & push_in)
   {
-    Gtk::TreeIter iter;
-    Gtk::TreePath dest_path;
-    Glib::RefPtr<Gtk::TreeSelection> selection;
-
     // Set default "return" values
     push_in = false; // not used
     x = 0;
     y = 0;
 
-    selection = m_tree->get_selection();
-    iter = selection->get_selected();
-    if (!iter)
+    Gtk::Widget * const focus_widget = this->get_focus();
+    if (!focus_widget)
+      return;
+    focus_widget->get_window()->get_origin(x, y);
+
+    Gtk::TreeView * const tree = dynamic_cast<Gtk::TreeView*>(
+                                   focus_widget);
+    if (!tree)
+      return;
+    const Glib::RefPtr<Gdk::Window> tree_area
+                                      = tree->get_bin_window();
+    if (!tree_area)
+      return;
+    tree_area->get_origin(x, y);
+
+    const Glib::RefPtr<Gtk::TreeSelection> selection
+                                             = tree->get_selection();
+    const std::list<Gtk::TreePath> selected_rows
+                                     = selection->get_selected_rows();
+    if (selected_rows.empty())
       return;
 
-    dest_path = m_store_sort->get_path (iter);
-
-    int pos_x = 0;
-    int pos_y = 0;
-
-    get_widget_screen_pos (*m_tree, pos_x, pos_y);
+    const Gtk::TreePath & dest_path = selected_rows.front();
+    const std::list<Gtk::TreeViewColumn *> columns
+                                             = tree->get_columns();
     Gdk::Rectangle cell_rect;
-    m_tree->get_cell_area (dest_path, **m_tree->get_columns().begin(), cell_rect);
+    tree->get_cell_area (dest_path, *columns.front(), cell_rect);
 
-    // Add 100 to x so it's not be at the far left
-    x = pos_x + cell_rect.get_x() + 100;
-    y = pos_y + cell_rect.get_y();
+    x += cell_rect.get_x();
+    y += cell_rect.get_y();
   }
 
 
-  // Walk the widget hiearchy to figure out
-  // where to position the context menu.
-  void NoteRecentChanges::get_widget_screen_pos(Gtk::Widget & widget, int & x, int & y)
-  {
-    int widget_x;
-    int widget_y;
-    Gtk::Window *win;
-
-    win = dynamic_cast<Gtk::Window*>(&widget);
-    if (win) {
-      win->get_position (widget_x, widget_y);
-    } 
-    else {
-      get_widget_screen_pos(*widget.get_parent(), x, y);
-
-      // Special case the TreeView because it adds
-      // too much since it's in a scrolled window.
-      if (&widget == m_tree) {
-        widget_x = 2;
-        widget_y = 2;
-      } 
-      else {
-        Gdk::Rectangle widget_rect = widget.get_allocation();
-        widget_x = widget_rect.get_x();
-        widget_y = widget_rect.get_y();
-      }
-    }
-
-    x += widget_x;
-    y += widget_y;
-  }
-
   Note::List NoteRecentChanges::get_selected_notes()
   {
     Note::List selected_notes;
@@ -1066,18 +1114,6 @@ namespace gnote {
       // Allow Escape to close the window
       on_close_window ();
       break;
-    case GDK_Menu:
-    {
-      // Pop up the context menu if a note is selected
-      Note::List selected_notes = get_selected_notes ();
-      if (!selected_notes.empty()) {
-        Gtk::Menu *menu = dynamic_cast<Gtk::Menu*>(ActionManager::obj().get_widget (
-                                                     "/MainWindowContextMenu"));
-        popup_context_menu_at_location (menu, 0, 0);
-      }
-
-      break;
-    }
     default:
       break;
     }
@@ -1328,11 +1364,22 @@ namespace gnote {
       if (notebook->get_tag()) {
         m_selected_tags.insert(notebook->get_tag());
       }
+      bool allow_edit = false;
       if (std::tr1::dynamic_pointer_cast<notebooks::SpecialNotebook>(notebook)) {
         am["DeleteNotebookAction"]->set_sensitive(false);
       } 
       else {
         am["DeleteNotebookAction"]->set_sensitive(true);
+        allow_edit = true;
+      }
+
+      Glib::ListHandle<Gtk::CellRenderer*> renderers = m_notebooksTree->get_column(0)->get_cell_renderers();
+      for (Glib::ListHandle<Gtk::CellRenderer*>::iterator renderer = renderers.begin();
+           renderer != renderers.end(); ++renderer) {
+        Gtk::CellRendererText *text_rederer = dynamic_cast<Gtk::CellRendererText*>(*renderer);
+        if (text_rederer) {
+          text_rederer->property_editable() = allow_edit;
+        }
       }
     }
       
@@ -1371,19 +1418,7 @@ namespace gnote {
     }
       
     // Look for the template note and create a new note
-    Note::Ptr templateNote = notebook->get_template_note ();
-    Note::Ptr note;
-      
-    note = m_manager.create ();
-    if (templateNote) {
-      // Use the body from the template note
-      std::string xmlContent = sharp::string_replace_first(templateNote->xml_content(), 
-                                                           templateNote->get_title(), 
-                                                           note->get_title());
-      note->set_xml_content(xmlContent);
-    }
-      
-    note->add_tag(notebook->get_tag());
+    Note::Ptr note = notebook->create_notebook_note ();
     note->get_window()->show ();
   }
     
@@ -1436,20 +1471,26 @@ namespace gnote {
   {
     if(ev->button == 3) {
       // third mouse button (right-click)
+      Gtk::TreeViewColumn * col = 0; // unused
+      Gtk::TreePath p;
+      int cell_x, cell_y;            // unused
+      const Glib::RefPtr<Gtk::TreeSelection> selection
+        = m_notebooksTree->get_selection();
+
+      if (m_notebooksTree->get_path_at_pos(ev->x, ev->y, p, col,
+                                           cell_x, cell_y)) {
+        selection->select(p);
+      }
+
       notebooks::Notebook::Ptr notebook = get_selected_notebook ();
       if (!notebook)
         return true; // Don't pop open a submenu
           
-      Gtk::TreePath p;
 
       bool rowClicked = true;
-      Gtk::TreeViewColumn * col = NULL; // unused
-      int cell_x, cell_y;               // unused
       if (m_notebooksTree->get_path_at_pos (ev->x, ev->y, p, col, cell_x, cell_y) == false) {
         rowClicked = false;
       }
-
-      Glib::RefPtr<Gtk::TreeSelection> selection = m_notebooksTree->get_selection();
       if (selection->count_selected_rows () == 0) {
         rowClicked = false;
       }
@@ -1475,15 +1516,12 @@ namespace gnote {
   bool NoteRecentChanges::on_notebooks_key_pressed(GdkEventKey * ev)
   {
     switch (ev->keyval) {
-    case GDK_Escape:
-      // Allow Escape to close the window
-      on_close_window ();
-      break;
     case GDK_Menu:
+    {
       // Pop up the context menu if a notebook is selected
       notebooks::Notebook::Ptr notebook = get_selected_notebook ();
       if (!notebook || std::tr1::dynamic_pointer_cast<notebooks::SpecialNotebook>(notebook))
-        return true; // Don't pop open a submenu
+        break; // Don't pop open a submenu
           
       Gtk::Menu *menu = dynamic_cast<Gtk::Menu *>(ActionManager::obj().get_widget (
                                                     "/NotebooksTreeContextMenu"));
@@ -1491,7 +1529,11 @@ namespace gnote {
 
       break;
     }
-    return false;
+    default:
+      break;
+    }
+
+    return false; // Let Escape be handled by the window.
   }
     
   void NoteRecentChanges::on_note_added_to_notebook (const Note & , 
@@ -1543,6 +1585,8 @@ namespace gnote {
     prefs.set<int> (Preferences::SEARCH_WINDOW_Y_POS, y);
     prefs.set<int> (Preferences::SEARCH_WINDOW_WIDTH, width);
     prefs.set<int> (Preferences::SEARCH_WINDOW_HEIGHT, height);
+    prefs.set<int> (Preferences::SEARCH_WINDOW_SPLITTER_POS, 
+                    m_hpaned.get_position());
   }
         
    
@@ -1553,7 +1597,7 @@ namespace gnote {
     int y = prefs.get<int> (Preferences::SEARCH_WINDOW_Y_POS);
     int width = prefs.get<int> (Preferences::SEARCH_WINDOW_WIDTH);
     int height = prefs.get<int> (Preferences::SEARCH_WINDOW_HEIGHT);
-          
+    int pos = prefs.get<int> (Preferences::SEARCH_WINDOW_SPLITTER_POS);
 
     if((width == 0) || (height == 0)) {
       return;
@@ -1561,6 +1605,9 @@ namespace gnote {
           
     set_default_size(width, height);
     move (x, y);
+    if(pos) {
+      m_hpaned.set_position(pos);
+    }
   }
 
 
